//
//  AppController.m
//  BSTwitterApp
//
//  Created by Brent Simmons on 4/26/07.
//  Copyright 2007 Brent Simmons. All rights reserved.
//

/*
This app is just a demo, mainly just to show some basic use of NSXMLDocument + WebKit + NSURLConnection.
It's not cool like Twitterrific, which is what I use. (I highly recommend Twitterrific.)

It doesn't post! It doesn't store your password in the keychain! It doesn't automatically refresh! It's useless! Just a demo! Don't worry, Craig!

To use the, type your username and password in the bottom, then click the Save button. Then click the Refresh button. (You have to type username and password every time you launch the app.)

API documentation: 
http://groups.google.com/group/twitter-development-talk/web/api-documentation
http://twitter.pbwiki.com/API-Docs */


#import "AppController.h"
#import "BSTwitterAPICall.h"
#import "BSTwitterParser.h"


@interface AppController (Forward)
- (void)_setAPICallInProgress:(BOOL)flag;
@end


@implementation AppController


#pragma mark XML to HTML

- (NSString *)_htmlStringWithTweet:(NSDictionary *)tweetDict {
	NSMutableString *s = [NSMutableString stringWithString:@""];
	[s appendString:@"<h2>"];
	if ([tweetDict objectForKey:BSAuthorURL]) {
		[s appendString:@"<a href=\""];
		[s appendString:[tweetDict objectForKey:BSAuthorURL]];
		[s appendString:@"\">"];
		}
	[s appendString:[tweetDict objectForKey:BSAuthorName]];
	if ([tweetDict objectForKey:BSAuthorURL])
		[s appendString:@"</a>"];
	[s appendString:@" ("];
	[s appendString:[tweetDict objectForKey:BSAuthorScreenName]];
	[s appendString:@")</h2>\n"];
	[s appendString:@"<p>"];
	if ([tweetDict objectForKey:BSAuthorImageURL]) {
		[s appendString:@"<img src=\""];
		[s appendString:[tweetDict objectForKey:BSAuthorImageURL]];
		[s appendString:@" align=\"right\" hspace=\"5\" alt=\"\" />"];
		}
	[s appendString:[tweetDict objectForKey:BSText]];
	[s appendString:@"<br />\nCreated at: "];
	[s appendString:[tweetDict objectForKey:BSCreatedAt]];
	return s;
	}


- (NSString *)_htmlStringWithXMLData:(NSData *)d {
	NSArray *tweets = [BSTwitterParser parsedArrayWithXMLData:d];
	NSMutableString *htmlString = [NSMutableString stringWithString:@""];
	int i;
	int ct = [tweets count];
	if (ct < 1)
		return htmlString;
	[htmlString appendString:@"<html><head><title>Tweets!</title></head><body>\n"];
	NSDictionary *oneTweet;
	for (i = 0; i < ct; i++) { /*for-loop, bite me*/
		oneTweet = [tweets objectAtIndex:i];
		[htmlString appendString:[self _htmlStringWithTweet:oneTweet]];
		}
	[htmlString appendString:@"</body></html>"];
	return htmlString;
	}


- (void)_updateWebViewWithTweets:(BSTwitterAPICall *)apiCall {
	NSLog([[[NSString alloc] initWithData:[apiCall downloadData] encoding:NSUTF8StringEncoding] autorelease]);
	NSString *htmlString = [self _htmlStringWithXMLData:[apiCall downloadData]];
	[[_webview mainFrame] loadHTMLString:htmlString baseURL:[NSURL URLWithString:@"http://twitter.com/"]];
	}


#pragma mark API Callback

- (void)apiCallDidComplete:(BSTwitterAPICall *)apiCall {
	if ([apiCall didSucceed])
		[self _updateWebViewWithTweets:apiCall];
	[apiCall autorelease];
	[self _setAPICallInProgress:NO];
	}


#pragma mark Accessors

- (void)_setUsername:(NSString *)s {
	[_username autorelease];
	_username = [s copy];
	}


- (void)_setPassword:(NSString *)s {
	[_password autorelease];
	_password = [s copy];
	}


- (void)_setAPICallInProgress:(BOOL)flag {
	_apiCallInProgress = flag;
	if (flag)
		[_progressIndicator startAnimation:self];
	else
		[_progressIndicator stopAnimation:self];
	}
	

#pragma mark Actions

- (IBAction)refreshFriendsTweets:(id)sender {
	if (!_username || !_password)
		return;
	[self _setAPICallInProgress:YES];
	BSTwitterAPICall *apiCall = [[BSTwitterAPICall alloc] initWithUsername:_username password:_password delegate:self callbackSelector:@selector(apiCallDidComplete:)];
	[apiCall refreshFriendsTweets];
	}


- (IBAction)saveUsernameAndPassword:(id)sender {
	[self _setUsername:[_usernameTextField stringValue]];
	[self _setPassword:[_passwordTextField stringValue]];
	[_passwordTextField setStringValue:@""];
	}

#pragma mark WebPolicyDelegate
 
- (void)webView:(WebView *)webView decidePolicyForNavigationAction:(NSDictionary *)actionInformation request:(NSURLRequest *)request frame:(WebFrame *)frame decisionListener:(id<WebPolicyDecisionListener>)listener {
	
	int navType = [[actionInformation objectForKey:WebActionNavigationTypeKey] intValue];
	if (navType == WebNavigationTypeLinkClicked) {
		[[NSWorkspace sharedWorkspace] openURL:[request URL]];
		[listener ignore];
		}
	else
		[listener use];
	}

@end
