//
//  BSTwitterAPICall.m
//  BSTwitterApp
//
//  Created by Brent Simmons on 4/26/07.
//  Copyright 2007 Brent Simmons. All rights reserved.
//

#import "BSTwitterAPICall.h"


@interface BSTwitterAPICall (Forward)
- (void)_setDownloadData:(NSMutableData *)data;
- (void)_setURLConnection:(NSURLConnection *)connection;
- (void)_handleFinishedDownloading;
@end


@implementation BSTwitterAPICall

#pragma mark Init

- (id)initWithUsername:(NSString *)username password:(NSString *)password delegate:(id)delegate callbackSelector:(SEL)aSelector {
	if (![super init])
		return nil;
	_username = [username copy];
	_password = [password copy];
	_delegate = delegate;
	_callbackSelector = aSelector;
	return self;
	}

#pragma mark Dealloc

- (void)dealloc {
	[_username release];
	[_password release];
	[_urlResponse release];
	[_urlConnection release];
	[_downloadData release];
	[super dealloc];
	}


#pragma mark Run API Call

- (void)refreshFriendsTweets {
	NSMutableURLRequest *urlRequest = [[[NSMutableURLRequest alloc] initWithURL:[NSURL URLWithString:@"http://twitter.com/statuses/friends_timeline.xml"]] autorelease];
	[urlRequest setCachePolicy:NSURLRequestReloadIgnoringCacheData];
	[urlRequest addValue:@"BSTwitterApp" forHTTPHeaderField:@"X-Twitter-Client"]; /* http://twitter.pbwiki.com/API-Docs#RequestHeaders */
	[self _setDownloadData:[NSMutableData dataWithLength:4096]];
	NSURLConnection *connection = [[[NSURLConnection alloc] initWithRequest:urlRequest delegate:self] autorelease];
	[self _setURLConnection:connection];
	}


- (BOOL)didSucceed {
	return _didSucceed;
	}


- (void)_setDidSucceed:(BOOL)flag {
	_didSucceed = flag;
	}


#pragma mark Downloading

- (void)_setURLConnection:(NSURLConnection *)connection {
	[_urlConnection autorelease];
	_urlConnection = [connection retain];
	}


- (NSMutableData *)downloadData {
	return _downloadData;
	}


- (void)_setDownloadData:(NSMutableData *)data {
	[_downloadData autorelease];
	_downloadData = [data retain];
	}


- (void)_setURLResponse:(NSURLResponse *)response {
	[_urlResponse autorelease];
	_urlResponse = [response retain];
	}


- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response {
	[_downloadData setLength:0];
	[self _setURLResponse:response];
	}
	

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
	[_downloadData appendData:data];
	}
	

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error {
	[self _setDidSucceed:NO];
	[self _handleFinishedDownloading];
	}
	
	
- (void)connectionDidFinishLoading:(NSURLConnection *)connection {
	[self _setDidSucceed:[(NSHTTPURLResponse *)_urlResponse statusCode] == 200];
	[self _handleFinishedDownloading];
	}


- (void)connection:(NSURLConnection *)connection didReceiveAuthenticationChallenge:(NSURLAuthenticationChallenge *)challenge {
	
	int previousFailureCount = [challenge previousFailureCount];
	
	if (previousFailureCount < 2) {
		NSURLCredential *newCredential = [NSURLCredential credentialWithUser:_username password:_password persistence:NSURLCredentialPersistencePermanent];
		[[challenge sender] useCredential:newCredential forAuthenticationChallenge:challenge];
		}
	else {
		[[challenge sender] cancelAuthenticationChallenge:challenge];
		[self _setDidSucceed:NO];	
		}	
	}


- (void)_handleFinishedDownloading {
	[_delegate performSelector:_callbackSelector withObject:self];
	}



@end
