//
//  BSTwitterParser.m
//	BSTwitterApp
//
//  Created by Brent Simmons on 4/26/07.
//  Copyright 2007 Brent Simmons. All rights reserved.
//

/*Given raw XML data, it generates an array of dictionaries. Each dictionary is a tweet.*/

#import "BSTwitterParser.h"

/*Keys to a tweet dictionary*/

NSString *BSCreatedAt = @"createdAt";
NSString *BSText = @"text";
NSString *BSAuthorName = @"authorName";
NSString *BSAuthorImageURL = @"authorImageURL";
NSString *BSAuthorURL = @"authorURL";
NSString *BSAuthorScreenName = @"authorScreenName";


@implementation BSTwitterParser


+ (void)_addAttributeWithNode:(NSXMLNode *)parentNode xPath:(NSString *)xPath toDictionary:(NSMutableDictionary *)dict withKey:(NSString *)key {
	NSError *error = nil;
	NSArray *nodes = [parentNode nodesForXPath:xPath error:&error];
	if (!nodes || [nodes count] < 1)
		return;
	NSXMLNode *node = [nodes objectAtIndex:0];
	if (node && [node stringValue])
		[dict setObject:[node stringValue] forKey:key];
	}


/*Example XML: This is a short version -- in reality there are multiple <status> items. (Just one <statuses>.)

<?xml version="1.0" encoding="UTF-8"?>
<statuses>
  <status>
    <created_at>Thu Apr 26 21:22:12 +0000 2007</created_at>
    <id>41122532</id>
    <text>Working on a demo app for tonight's Xcoders meeting. It will demo NSXMLDocument/WebKit/NSURLConnection in the context of the Twitter API.</text>
    <user>
      <id>652293</id>
      <name>Brent Simmons</name>
      <screen_name>brentsimmons</screen_name>
      <location>Seattle</location>
      <description>Programmer.</description>
      <profile_image_url>http://assets3.twitter.com/system/user/profile_image/652293/normal/nnwchaticon.png?1171964917</profile_image_url>
      <url>http://inessential.com/</url>
      <protected>false</protected>
    </user>
  </status>
</statuses>

*/

+ (NSDictionary *)_parsedObjectWithNode:(NSXMLNode *)node {
	NSMutableDictionary *d = [NSMutableDictionary dictionaryWithCapacity:6];
	[self _addAttributeWithNode:node xPath:@".//created_at" toDictionary:d withKey:BSCreatedAt];
	[self _addAttributeWithNode:node xPath:@".//text" toDictionary:d withKey:BSText];
	[self _addAttributeWithNode:node xPath:@".//user/name" toDictionary:d withKey:BSAuthorName];
	[self _addAttributeWithNode:node xPath:@".//user/profile_image_url" toDictionary:d withKey:BSAuthorImageURL];
	[self _addAttributeWithNode:node xPath:@".//user/url" toDictionary:d withKey:BSAuthorURL];
	[self _addAttributeWithNode:node xPath:@".//user/screen_name" toDictionary:d withKey:BSAuthorScreenName];
	return d;
	}


+ (NSArray *)_parsedObjectsWithNodes:(NSArray *)nodes {
	if (!nodes)
		return nil;
	NSMutableArray *tempArray = [NSMutableArray arrayWithCapacity:[nodes count]];
	int i;
	int ct = [nodes count];
	for (i = 0; i < ct; i++)
		[tempArray addObject:[self _parsedObjectWithNode:[nodes objectAtIndex:i]]];
	return tempArray;
	}


+ (NSArray *)_parsedObjectsWithXMLDocument:(NSXMLDocument *)doc {
	NSError *error = nil;
	NSArray *nodes = [[doc rootElement] nodesForXPath:@".//status" error:&error];
	return [self _parsedObjectsWithNodes:nodes];
	}


+ (NSArray *)parsedArrayWithXMLData:(NSData *)d {
	if (!d)
		return nil;
	NSError *error = nil;
	NSXMLDocument *doc = [[[NSXMLDocument alloc] initWithData:d options:0 error:&error] autorelease];
	if (!doc && error)
		doc = [[[NSXMLDocument alloc] initWithData:d options:NSXMLDocumentTidyXML error:&error] autorelease];
	if (!doc)
		return nil;
	return [self _parsedObjectsWithXMLDocument:doc];
	}
	

@end


